@@ -2,6 +2,33 @@
 # -*- coding: utf-8 -*-
 
 from __future__ import absolute_import, division, print_function
+
+__copyright__ = """
+Copyright (C) 2009-2017 Andreas Kloeckner
+Copyright (C) 2014-2017 Aaron Meurer
+"""
+
+__license__ = """
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
+"""
+
+
 import urwid
 import bdb
 import gc
@@ -17,7 +44,7 @@ from pudb.py3compat import PY3, raw_input, execfile
 CONFIG = load_config()
 save_config(CONFIG)
 
-HELP_TEXT = """\
+HELP_TEXT = r"""\
 Welcome to PuDB, the Python Urwid debugger.
 -------------------------------------------
 
@@ -79,7 +106,7 @@ Sidebar-related (active in sidebar):
     [/] - grow/shrink relative size of active sidebar box
 
 Keys in variables list:
-    \ - expand/collapse
+    \/space - expand/collapse
     t/r/s/c - show type/repr/str/custom for this variable
     h - toggle highlighting
     @ - toggle repetition at top
@@ -132,8 +159,10 @@ OTHER DEALINGS IN THE SOFTWARE.
 # {{{ debugger interface
 
 class Debugger(bdb.Bdb):
-    def __init__(self, stdin=None, stdout=None, term_size=None, steal_output=False):
-        bdb.Bdb.__init__(self)
+    def __init__(self, stdin=None, stdout=None, term_size=None, steal_output=False,
+            **kwargs):
+        # Pass remaining kwargs to python debugger framework
+        bdb.Bdb.__init__(self, **kwargs)
         self.ui = DebuggerUI(self, stdin=stdin, stdout=stdout, term_size=term_size)
         self.steal_output = steal_output
 
@@ -382,10 +411,8 @@ class Debugger(bdb.Bdb):
 
     def user_return(self, frame, return_value):
         """This function is called when a return trap is set here."""
-        if frame.f_code.co_name == '<module>':
-            return
-
-        frame.f_locals['__return__'] = return_value
+        if frame.f_code.co_name != '<module>':
+            frame.f_locals['__return__'] = return_value
 
         if self._wait_for_mainpyfile:
             if (self.mainpyfile != self.canonic(frame.f_code.co_filename)
@@ -406,11 +433,19 @@ class Debugger(bdb.Bdb):
             self.interaction(frame, exc_tuple)
 
     def _runscript(self, filename):
-        # Start with fresh empty copy of globals and locals and tell the script
-        # that it's being run as __main__ to avoid scripts being able to access
-        # the debugger's namespace.
-        globals_ = {"__name__": "__main__", "__file__": filename}
-        locals_ = globals_
+        # Provide separation from current __main__, which is likely
+        # pudb.__main__ run.  Preserving its namespace is not important, and
+        # having the script share it ensures that, e.g., pickle can find
+        # types defined there:
+        # https://github.com/inducer/pudb/issues/331
+
+        import __main__
+        __main__.__dict__.clear()
+        __main__.__dict__.update({
+            "__name__": "__main__",
+            "__file__": filename,
+            "__builtins__": __builtins__,
+            })
 
         # When bdb sets tracing, a number of call and line events happens
         # BEFORE debugger even reaches user's code (and the exact sequence of
@@ -429,7 +464,8 @@ class Debugger(bdb.Bdb):
         from pudb import set_interrupt_handler
         set_interrupt_handler()
 
-        self.run(statement, globals=globals_, locals=locals_)
+        # Implicitly runs in the namespace of __main__.
+        self.run(statement)
 
 # }}}
 
@@ -572,7 +608,7 @@ class FileSourceCodeProvider(SourceCodeProvider):
             lines = getlines(self.file_name)
             return format_source(
                     debugger_ui, list(decode_lines(lines)), set(breakpoints))
-        except:
+        except Exception:
             from pudb.lowlevel import format_exception
             debugger_ui.message("Could not load source file '%s':\n\n%s" % (
                 self.file_name, "".join(format_exception(sys.exc_info()))),
@@ -589,10 +625,8 @@ class DirectSourceCodeProvider(SourceCodeProvider):
     def __eq__(self, other):
         return (
                 type(self) == type(other)
-                and
-                self.function_name == other.function_name
-                and
-                self.code is other.code)
+                and self.function_name == other.function_name
+                and self.code is other.code)
 
     def identifier(self):
         return "<source code of function %s>" % self.function_name
@@ -745,10 +779,13 @@ class DebuggerUI(FrameVarInfoKeeper):
         def change_var_state(w, size, key):
             var, pos = self.var_list._w.get_focus()
 
+            if var is None:
+                return
+
             iinfo = self.get_frame_var_info(read_only=False) \
                     .get_inspect_info(var.id_path, read_only=False)
 
-            if key == "\\":
+            if key == "\\" or key == ' ':
                 iinfo.show_detail = not iinfo.show_detail
             elif key == "t":
                 iinfo.display_type = "type"
@@ -833,10 +870,10 @@ class DebuggerUI(FrameVarInfoKeeper):
                     "Show methods", iinfo.show_methods)
 
             lb = urwid.ListBox(urwid.SimpleListWalker(
-                id_segment +
-                rb_grp_show + [urwid.Text("")] +
-                rb_grp_access + [urwid.Text("")] +
-                [
+                id_segment
+                + rb_grp_show + [urwid.Text("")]
+                + rb_grp_access + [urwid.Text("")]
+                + [
                     wrap_checkbox,
                     expanded_checkbox,
                     highlighted_checkbox,
@@ -900,6 +937,7 @@ class DebuggerUI(FrameVarInfoKeeper):
                 self.update_var_view()
 
         self.var_list.listen("\\", change_var_state)
+        self.var_list.listen(" ", change_var_state)
         self.var_list.listen("t", change_var_state)
         self.var_list.listen("r", change_var_state)
         self.var_list.listen("s", change_var_state)
@@ -916,6 +954,9 @@ class DebuggerUI(FrameVarInfoKeeper):
         self.var_list.listen("[", partial(change_rhs_box, 'variables', 0, -1))
         self.var_list.listen("]", partial(change_rhs_box, 'variables', 0, 1))
 
+        self.var_list.listen("j", self.rhs_scroll_down)
+        self.var_list.listen("k", self.rhs_scroll_up)
+
         # }}}
 
         # {{{ stack listeners
@@ -941,6 +982,9 @@ class DebuggerUI(FrameVarInfoKeeper):
         self.stack_list.listen("[", partial(change_rhs_box, 'stack', 1, -1))
         self.stack_list.listen("]", partial(change_rhs_box, 'stack', 1, 1))
 
+        self.stack_list.listen("j", self.rhs_scroll_down)
+        self.stack_list.listen("k", self.rhs_scroll_up)
+
         # }}}
 
         # {{{ breakpoint listeners
@@ -1063,6 +1107,8 @@ class DebuggerUI(FrameVarInfoKeeper):
         self.bp_list.listen("[", partial(change_rhs_box, 'breakpoints', 2, -1))
         self.bp_list.listen("]", partial(change_rhs_box, 'breakpoints', 2, 1))
 
+        self.bp_list.listen("j", self.rhs_scroll_down)
+        self.bp_list.listen("k", self.rhs_scroll_up)
         # }}}
 
         # {{{ source listeners
@@ -1143,13 +1189,14 @@ class DebuggerUI(FrameVarInfoKeeper):
             _, line = self.source.get_focus()
 
             lineno_edit = urwid.IntEdit([
-                ("label", "Line number: ")
-                ], line+1)
+                ("label", "Go to Line   :")
+                ], None)
 
             if self.dialog(
                     urwid.ListBox(urwid.SimpleListWalker([
                         labelled_value("File :",
                             self.source_code_provider.identifier()),
+                        labelled_value("Current Line :", line+1),
                         urwid.AttrMap(lineno_edit, "value")
                         ])),
                     [
@@ -1267,6 +1314,8 @@ class DebuggerUI(FrameVarInfoKeeper):
             def mod_exists(mod):
                 if not hasattr(mod, "__file__"):
                     return False
+                if mod.__file__ is None:
+                    return False
                 filename = mod.__file__
 
                 base, ext = splitext(filename)
@@ -1346,7 +1395,7 @@ class DebuggerUI(FrameVarInfoKeeper):
                         new_mod_name = filt_edit.get_edit_text()
                         try:
                             __import__(str(new_mod_name))
-                        except:
+                        except Exception:
                             from pudb.lowlevel import format_exception
 
                             self.message("Could not import module '%s':\n\n%s" % (
@@ -1536,12 +1585,16 @@ class DebuggerUI(FrameVarInfoKeeper):
             sys.stdin = None
             sys.stderr = sys.stdout = StringIO()
             try:
-                # Don't use cmdline_get_namespace() here, it breaks things in
-                # Python 2 (issue #166).
-                eval(compile(cmd, "<pudb command line>", 'single'),
-                        self.debugger.curframe.f_globals,
-                        self.debugger.curframe.f_locals)
-            except:
+                # Don't use cmdline_get_namespace() here in Python 2, as it
+                # breaks things (issue #166).
+                if PY3:
+                    eval(compile(cmd, "<pudb command line>", 'single'),
+                         cmdline_get_namespace())
+                else:
+                    eval(compile(cmd, "<pudb command line>", 'single'),
+                         self.debugger.curframe.f_globals,
+                         self.debugger.curframe.f_locals)
+            except Exception:
                 tp, val, tb = sys.exc_info()
 
                 import traceback
@@ -1779,7 +1832,7 @@ class DebuggerUI(FrameVarInfoKeeper):
                         from os.path import expanduser
                         execfile(
                                 expanduser(CONFIG["shell"]), shell.custom_shell_dict)
-                except:
+                except Exception:
                     print("Error when importing custom shell:")
                     from traceback import print_exc
                     print_exc()
@@ -1855,11 +1908,9 @@ class DebuggerUI(FrameVarInfoKeeper):
                 CONFIG["display"] == "curses"
                 or (
                     CONFIG["display"] == "auto"
-                    and
-                    not (
+                    and not (
                         os.environ.get("TERM", "").startswith("xterm")
-                        or
-                        os.environ.get("TERM", "").startswith("rxvt")
+                        or os.environ.get("TERM", "").startswith("rxvt")
                     )))
 
         if (want_curses_display
@@ -1885,7 +1936,7 @@ class DebuggerUI(FrameVarInfoKeeper):
         if curses:
             try:
                 curses.setupterm()
-            except:
+            except Exception:
                 # Something went wrong--oh well. Nobody will die if their
                 # 256 color support breaks. Just carry on without it.
                 # https://github.com/inducer/pudb/issues/78
@@ -1910,6 +1961,13 @@ class DebuggerUI(FrameVarInfoKeeper):
     # }}}
 
     # {{{ UI helpers
+    def rhs_scroll_down(self, w, size, key):
+        if key == 'j' and CONFIG['jk_sidebar_scroll']:
+            w.keypress(size, "down")
+
+    def rhs_scroll_up(self, w, size, key):
+        if key == 'k' and CONFIG['jk_sidebar_scroll']:
+            w.keypress(size, "up")
 
     def translate_ui_stack_index(self, index):
         # note: self-inverse
@@ -2105,7 +2163,7 @@ class DebuggerUI(FrameVarInfoKeeper):
                 self.message("Package 'pygments' not found. "
                         "Syntax highlighting disabled.")
 
-        WELCOME_LEVEL = "e033"  # noqa
+        WELCOME_LEVEL = "e035"  # noqa
         if CONFIG["seen_welcome"] < WELCOME_LEVEL:
             CONFIG["seen_welcome"] = WELCOME_LEVEL
             from pudb import VERSION
@@ -2122,6 +2180,17 @@ class DebuggerUI(FrameVarInfoKeeper):
                     "(invoked by hitting '?' after this message) should get you "
                     "on your way.\n"
 
+                    "\nChanges in version 2019.1:\n\n"
+                    "- Allow 'space' as a key to expand variables (Enrico Troeger)\n"
+                    "- Have a persistent setting on variable visibility (Enrico Troeger)\n"
+                    "- Enable/partially automate opening the debugger in another \n"
+                    "  terminal (Anton Barkovsky)\n"
+                    "- Make sidebar scrollable with j/k (Clayton Craft)\n"
+                    "- Bug fixes.\n"
+
+                    "\nChanges in version 2018.1:\n\n"
+                    "- Bug fixes.\n"
+
                     "\nChanges in version 2017.1.4:\n\n"
                     "- Bug fixes.\n"
 
@@ -2404,7 +2473,7 @@ class DebuggerUI(FrameVarInfoKeeper):
             if code.co_argcount and code.co_varnames[0] == "self":
                 try:
                     class_name = frame.f_locals["self"].__class__.__name__
-                except:
+                except Exception:
                     pass
 
             return StackFrame(frame is self.debugger.curframe,
