@@ -106,11 +106,14 @@ def load_config():
     conf_dict.setdefault("custom_shell", "")
 
     conf_dict.setdefault("wrap_variables", True)
+    conf_dict.setdefault("default_variables_access_level", "public")
 
     conf_dict.setdefault("display", "auto")
 
     conf_dict.setdefault("prompt_on_quit", True)
 
+    conf_dict.setdefault("jk_sidebar_scroll", False)
+
     def normalize_bool_inplace(name):
         try:
             if conf_dict[name].lower() in ["0", "false", "off"]:
@@ -123,6 +126,7 @@ def load_config():
     normalize_bool_inplace("line_numbers")
     normalize_bool_inplace("wrap_variables")
     normalize_bool_inplace("prompt_on_quit")
+    normalize_bool_inplace("jk_sidebar_scroll")
 
     return conf_dict
 
@@ -158,11 +162,14 @@ def edit_config(ui, conf_dict):
 
     def _update_line_numbers():
         for sl in ui.source:
-                sl._invalidate()
+            sl._invalidate()
 
     def _update_prompt_on_quit():
         pass
 
+    def _update_jk_sidebar_scroll():
+        pass
+
     def _update_current_stack_frame():
         ui.update_stack()
 
@@ -171,6 +178,9 @@ def edit_config(ui, conf_dict):
         pudb.var_view.custom_stringifier_dict = {}
         ui.update_var_view()
 
+    def _update_default_variables_access_level():
+        ui.update_var_view()
+
     def _update_wrap_variables():
         ui.update_var_view()
 
@@ -198,6 +208,11 @@ def edit_config(ui, conf_dict):
             conf_dict.update(new_conf_dict)
             _update_prompt_on_quit()
 
+        elif option == "jk_sidebar_scroll":
+            new_conf_dict["jk_sidebar_scroll"] = not check_box.get_state()
+            conf_dict.update(new_conf_dict)
+            _update_jk_sidebar_scroll()
+
         elif option == "current_stack_frame":
             # only activate if the new state of the radio button is 'on'
             if new_state:
@@ -213,6 +228,13 @@ def edit_config(ui, conf_dict):
 
                 conf_dict.update(stringifier=newvalue)
                 _update_stringifier()
+
+        elif option == "default_variables_access_level":
+            # only activate if the new state of the radio button is 'on'
+            if new_state:
+                conf_dict.update(default_variables_access_level=newvalue)
+                _update_default_variables_access_level()
+
         elif option == "wrap_variables":
             new_conf_dict["wrap_variables"] = not check_box.get_state()
             conf_dict.update(new_conf_dict)
@@ -233,6 +255,11 @@ def edit_config(ui, conf_dict):
             bool(conf_dict["prompt_on_quit"]), on_state_change=_update_config,
                 user_data=("prompt_on_quit", None))
 
+    cb_jk_sidebar_scroll = urwid.CheckBox("Use j/k keys for scrolling in "
+                                          "sidebar",
+            bool(conf_dict["jk_sidebar_scroll"]), on_state_change=_update_config,
+                user_data=("jk_sidebar_scroll", None))
+
     # {{{ shells
 
     shell_info = urwid.Text("This is the shell that will be "
@@ -340,6 +367,26 @@ def edit_config(ui, conf_dict):
 
     # }}}
 
+    # {{{ variables access level
+
+    default_variables_access_level_opts = ["public", "private", "all"]
+    default_variables_access_level_rb_group = []
+    default_variables_access_level_info = urwid.Text(
+            "Set the default attribute visibility "
+            "of variables in the variables list.\n"
+            "\nNote that you can change this option on "
+            "a per-variable basis by selecting the "
+            "variable and pressing '*'.")
+    default_variables_access_level_rbs = [
+            urwid.RadioButton(default_variables_access_level_rb_group, name,
+                conf_dict["default_variables_access_level"] == name,
+                on_state_change=_update_config,
+                user_data=("default_variables_access_level", name))
+            for name in default_variables_access_level_opts
+            ]
+
+    # }}}
+
     # {{{ wrap variables
 
     cb_wrap_variables = urwid.CheckBox("Wrap variables",
@@ -382,6 +429,9 @@ def edit_config(ui, conf_dict):
             + [urwid.AttrMap(urwid.Text("\nPrompt on quit:\n"), "group head")]
             + [cb_prompt_on_quit]
 
+            + [urwid.AttrMap(urwid.Text("\nKeys:\n"), "group head")]
+            + [cb_jk_sidebar_scroll]
+
             + [urwid.AttrMap(urwid.Text("\nShell:\n"), "group head")]
             + [shell_info]
             + shell_rbs
@@ -397,6 +447,11 @@ def edit_config(ui, conf_dict):
             + [stringifier_info]
             + stringifier_rbs
 
+            + [urwid.AttrMap(urwid.Text("\nVariables Attribute Visibility:\n"),
+                "group head")]
+            + [default_variables_access_level_info]
+            + default_variables_access_level_rbs
+
             + [urwid.AttrMap(urwid.Text("\nWrap Variables:\n"), "group head")]
             + [cb_wrap_variables]
             + [wrap_variables_info]
