@@ -45,7 +45,6 @@ if PY3:
 else:
     ELLIPSIS = unicode('…', 'utf-8')  # noqa: F821
 
-from pudb.debugger import CONFIG
 from pudb.ui_tools import text_width
 # }}}
 
@@ -68,11 +67,14 @@ class FrameVarInfo(object):
 
 class InspectInfo(object):
     def __init__(self):
+        # Do not globalize: cyclic import
+        from pudb.debugger import CONFIG
+
         self.show_detail = False
         self.display_type = CONFIG["stringifier"]
         self.highlighted = False
         self.repeated_at_top = False
-        self.access_level = "public"
+        self.access_level = CONFIG["default_variables_access_level"]
         self.show_methods = False
         self.wrap = CONFIG["wrap_variables"]
 
@@ -119,6 +121,9 @@ class VariableWidget(urwid.FlowWidget):
         self.attr_prefix = attr_prefix or "var"
         self.watch_expr = watch_expr
         if iinfo is None:
+            # Do not globalize: cyclic import
+            from pudb.debugger import CONFIG
+
             self.wrap = CONFIG["wrap_variables"]
         else:
             self.wrap = iinfo.wrap
@@ -167,7 +172,8 @@ class VariableWidget(urwid.FlowWidget):
         if self.wrap:
             text = self._get_text(size)
 
-            extralabel_full, extralabel_rem = divmod(text_width(var_label[maxcol:]), maxcol)
+            extralabel_full, extralabel_rem = divmod(
+                    text_width(var_label[maxcol:]), maxcol)
             totallen = sum([text_width(i) for i in text])
             labellen = (
                     len(self.prefix)  # Padding of first line
@@ -190,6 +196,8 @@ class VariableWidget(urwid.FlowWidget):
 
             return make_canvas(text, attr, maxcol, apfx+"value")
 
+        lprefix = len(self.prefix)
+
         if self.value_str is not None:
             if self.var_label is not None:
                 if len(self.prefix) + text_width(self.var_label) > self.SIZE_LIMIT:
@@ -197,13 +205,15 @@ class VariableWidget(urwid.FlowWidget):
                     text = [self.prefix + self.var_label,
                             self.prefix+"  " + self.value_str]
 
-                    attr = [[(apfx+"label", len(self.prefix)+text_width(self.var_label))],
-                            [(apfx+"value", len(self.prefix)+2+text_width(self.value_str))]]
+                    attr = [
+                        [(apfx+"label", lprefix+text_width(self.var_label))],
+                        [(apfx+"value", lprefix+2+text_width(self.value_str))]
+                        ]
                 else:
                     text = [self.prefix + self.var_label + ": " + self.value_str]
 
                     attr = [[
-                            (apfx+"label", len(self.prefix)+text_width(self.var_label)+2),
+                            (apfx+"label", lprefix+text_width(self.var_label)+2),
                             (apfx+"value", text_width(self.value_str)),
                             ]]
             else:
@@ -216,7 +226,7 @@ class VariableWidget(urwid.FlowWidget):
         else:
             text = [self.prefix + self.var_label]
 
-            attr = [[(apfx+"label", len(self.prefix) + text_width(self.var_label)), ]]
+            attr = [[(apfx+"label", lprefix + text_width(self.var_label)), ]]
 
         # Ellipses to show text was cut off
         #encoding = urwid.util.detected_encoding
